.bubble-shape {
  position: relative;
  &:before {
    content: '';
    position: absolute;
    width: 210px;
    height: 210px;
    border-radius: 50%;
    top: -125px;
    right: -15px;
    opacity: 0.5;
  }
  &:after {
    content: '';
    position: absolute;
    width: 210px;
    height: 210px;
    border-radius: 50%;
    top: -85px;
    right: -95px;
  }
}

.z-1 {
  z-index: 1;
  position: relative;
}
.bubble-shape-sm {
  position: relative;
  &::before {
    content: '';
    position: absolute;
    width: 210px;
    height: 210px;
    border-radius: 50%;
    top: -160px;
    right: -130px;
  }
  &::after {
    content: '';
    position: absolute;
    width: 210px;
    height: 210px;
    border-radius: 50%;
    top: -30px;
    right: -180px;
  }
}

.rounded-square {
  width: 20px;
  height: 20px;
}
